@@ -22,12 +22,12 @@ from extend_me import Extensible
 from psycopg2 import OperationalError
 from datetime import datetime
 from docker import APIClient as Client
-from docker.errors import APIError, NullResource, NotFound, ContainerError, ImageNotFound
+from docker.errors import APIError, NullResource, NotFound, ImageNotFound
 from deployv.base.extensions_core import events, load_extensions
 from deployv.base import postgresv, nginxv, errors, clonev
 from deployv.helpers import (utils, container, database_helper, sender_message,
                              build_helper, json_helper, git_helper)
-from deployv.instance import instancev, ODOO_BINARY
+from deployv.instance import instancev
 from deployv.extensions.checkers import InstallTestRepo
 from transitions import State, Machine
 from tempfile import mkdtemp
@@ -370,7 +370,8 @@ class CommandV(Extensible):
         self.sender.send_message('Executing final steps.')
         res = {'command': 'post_process'}
         post_process = ['cloc_report', 'list_packages']
-        if self.__config.instance_config.get('instance_type') == 'updates':
+        if (self.__config.instance_config.get('instance_type') == 'updates' and
+                self.__config.instance_config.get('auto_compare')):
             post_process.append('compare_databases')
         for command in post_process:
             if not hasattr(self, command):
@@ -701,7 +702,7 @@ class CommandV(Extensible):
                                'menus': menus_diff,
                                'views': views_diff,
                                'fields': fields_diff}})
-        self.sender.send_message('Data compared')
+        self.sender.send_message('Data compared', body=res)
         return res
 
     @utils.traceback_docker_error
@@ -715,13 +716,10 @@ class CommandV(Extensible):
         :returns: The result from :meth:`deployv.instance.instancev.run_and_log`.
         :rtype: dict
         """
+        self.__config.check_config('updatedb')
+        res = {'command': 'updatedb'}
         db_name = db_name or self.__config.instance_config.get('config').get('db_name')
         self.sender.send_message('Updating database "{}".'.format(db_name))
-        self.__config.check_config('updatedb')
-        update_modules = self.__config.instance_config.get('update_module', ["all"])
-        if not isinstance(update_modules, list):
-            update_modules = update_modules.split(",")
-        res = {'command': 'updatedb'}
         try:
             self.__instance_manager.stop_instance()
         except APIError as error:
@@ -730,32 +728,13 @@ class CommandV(Extensible):
             res.update({'error': error_msg})
             self.send_message_error(res)
             return res
-        retry = 0
-        res.update({"attachments": [], "result": {"update_logs": []}})
-        for module in update_modules:
-            while retry <= 1:
-                res_update = self.__instance_manager.update_db(module, db_name)
-                if res_update.get('errors'):
-                    if retry < 1:
-                        logger.info(
-                            'Some error shown in the update log, updating again')
-                        retry = retry + 1
-                    else:
-                        logger.error('The instance was not updated properly, check logfile: %s',
-                                     res_update.get('log_file'))
-                        res['result']['update_logs'].append(res_update)
-                        break
-                else:
-                    res['result']['update_logs'].append(res_update)
-                    break
+        update_res = self.instance_manager.update_db(db_name)
+        res.update(update_res)
+        if res.get('error'):
+            logger.error(res['error'])
+            self.send_message_error(res)
+            return res
         self.__instance_manager.start_instance()
-        for log in res["result"]["update_logs"]:
-            res["attachments"].append(
-                {
-                    'file_name': os.path.basename(log["log_file"]),
-                    'file': utils.generate_attachment(log["log_file"]),
-                    'type': 'text/plain'
-                })
         res['result'].update({'database_name': db_name})
         install_module = self.__config.instance_config.get('install_module')
         if install_module:
@@ -765,7 +744,7 @@ class CommandV(Extensible):
                 'install_module': install_res
             })
             if not res.get('attachments', False):
-                res.update({'attachments': list()})
+                res.update({'attachments': []})
             res.get('attachments').append(
                 {
                     'file_name': os.path.basename(install_res.get('log_file')),
@@ -1000,7 +979,7 @@ class CommandV(Extensible):
         postgres.create(db_name)
         create_date = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
         self.instance_manager.create_extension_unaccent(db_name)
-        self.instance_manager.update_db('all', db_name)
+        self.instance_manager.update_db(db_name, ['all'])
         res.get('result').update({'database_name': db_name, 'create_date': create_date})
         params = {'database.generated_at': create_date,
                   'database.type': self.instance_manager.instance_type}
@@ -1236,7 +1215,7 @@ class CommandV(Extensible):
         # Dump backup
         db_name = self.__config.instance_config.get('config', {}).get('db_name')
         info_msg = 'Generating backup for "{}" with DB "{}"'.format(container_name, db_name)
-        self.sender.send_message(info_msg)
+        self.sender.send_message('Generating backup')
         logger.info(info_msg)
         self.__config.reason = self.__config.reason or 'preupdate'
         # Change the cformat to False in order to receive the dict of files only
@@ -1266,103 +1245,34 @@ class CommandV(Extensible):
         :type container_name: str
         :type maintenance_container: str
         """
-        stable_image = self.__config.container_config.get('image_name')
-        manager = self.instance_manager
-        config = self.__config
-        res = {'result': {'update_logs': []}, 'attachments': []}
-        # Get the `version` store in the environment of stable image.
-        variables = manager.get_env_from_image(stable_image)
-        odoo_bin = ODOO_BINARY[variables.get('version')]
-        # Update modules
-        modules = config.instance_config.get('update_module')
-        container_cfg = config.container_config
-        working_folder = os.path.join(container_cfg.get("working_folder"), container_name)
-        volumes = container_cfg.get('volumes')
-        binded_volumes = container.generate_binds(volumes, working_folder)
-        container_options = {
-            'user': 'odoo',
-            'volumes': binded_volumes,
-            'remove': True,
-        }
-        db_config = config.instance_config.get('config') or {}
-        base_command = (
-            '{odoo} -c {rc} -d {db_name} -u {{module}} --logfile {{log}} --stop-after-init '
-            '-r {db_user} -w {db_password} --db_host {db_host} --db_port {db_port}'.format(
-                odoo=os.path.join('/home/odoo/instance/odoo', odoo_bin),
-                rc='/home/odoo/.openerp_serverrc', **db_config))
-        msg = 'Updating modules "{mod}" in "{db}" using "{img}" with options "{ops}"'.format(
-            mod=", ".join(modules), db=db_config.get('db_name'), img=stable_image,
-            ops=container_options)
-        logger.info(msg)
+        modules = self.__config.instance_config.get('update_module')
+        db_config = self.instance_manager.db_config
+        msg = 'Updating modules "{mod}" in "{db}" using "{img}""'.format(
+            mod=", ".join(modules), db=db_config.get('db_name'),
+            img=self.__config.container_config.get('image_name'))
         self.sender.send_message(msg)
-        pg_shell = postgresv.PostgresShell(utils.odoo2postgres(manager.db_config))
-        for module in modules:
-            for attempt in range(2):
-                log_file = 'update_{mod}_{date}.log'.format(mod=module, date=utils.get_strtime())
-                log_path = os.path.join('/tmp/deployvlogs', log_file)
-                local_path = os.path.expanduser(os.path.join(working_folder, log_path.strip('/')))
-                command = base_command.format(module=module, log=log_path)
-                self.sender.send_message('Updating "{}"'.format(module))
-                logger.info('Updating "%s"', module)
-                # Terminate connections before updating
-                pg_shell.terminate_sessions(db_config['db_name'], db_config.get('db_owner'),
-                                            db_config.get('db_owner_passwd'))
-                try:
-                    manager.cli2.containers.run(stable_image, command, **container_options)
-                except (ContainerError, APIError) as error:
-                    res.update({'error': 'Error updating "{mod}": {err}'.format(
-                        mod=module, err=str(error))})
-                    parsed_log = self._parse_update_production_log(local_path, module)
-                    summary = parsed_log.get('summary')
-                    if summary:
-                        res['result']['update_logs'].append(summary)
-                        res['attachments'].append(parsed_log.get('attachment'))
-                    # Remove maintenance container
-                    if maintenance_container:
-                        manager.remove_container(maintenance_container)
-                    return res
-                logger.info('Log was saved to "%s"', local_path)
-                parsed_log = self._parse_update_production_log(local_path, module)
-                if parsed_log.get('error') and not parsed_log.get('summary'):
-                    # Return the error message if the log file couldn't be generated
-                    res.update(parsed_log)
-                    # Remove maintenance container
-                    if maintenance_container:
-                        manager.remove_container(maintenance_container)
-                    return res
-                summary = parsed_log.get('summary')
-                if parsed_log.get('error'):
-                    if attempt:
-                        res.update({'error': parsed_log.get('error')})
-                        res['result']['update_logs'].append(summary)
-                        res['attachments'].append(parsed_log.get('attachment'))
-                        # Remove maintenance container
-                        if maintenance_container:
-                            manager.remove_container(maintenance_container)
-                        return res
-                    logger.info('Trying again')
-                    continue
-                res['result']['update_logs'].append(summary)
-                res['attachments'].append(parsed_log.get('attachment'))
-                break
-        # Remove maintenance container
+        res = self.instance_manager.update_db(db_config.get('db_name'))
+        # Remove maintenance container despite if the process finished successfully or not.
         if maintenance_container:
-            manager.remove_container(maintenance_container)
+            self.instance_manager.remove_container(maintenance_container)
+        if res.get('error'):
+            return res
         # Create container
-        self.sender.send_message('Starting container and instance in "{}"'.format(container_name))
+        self.sender.send_message('Starting instance')
         logger.info('Starting container and instance in "%s"', container_name)
-        manager.start_odoo_container()
+        self.instance_manager.start_odoo_container()
         # Start instance
         logger.info('Waiting for Odoo to start in "%s"', container_name)
-        started = manager.ensure_instance_status(ensure_running=True,
-                                                 max_tries=config.status_checks)
+        started = self.instance_manager.ensure_instance_status(
+            ensure_running=True, max_tries=self.__config.status_checks)
         if not started:
             res.update({'error': 'Instance not running in "{container}"'.format(
                 container=container_name
             )})
             return res
+        self.sender.send_message('Production instance started')
         res['result'].update({
-            'modules': modules,
+            'modules': modules or [],
             'container_name': container_name,
         })
         return res
@@ -1389,47 +1299,19 @@ class CommandV(Extensible):
         manager = self.__instance_manager
         res = {'result': {}}
         # Compress backup
-        self.sender.send_message('Compressing backup for "{}" in "{}"'.format(
-            container_name, directory))
+        self.sender.send_message('Compressing production backup')
         logger.info('Compressing backup for "%s" in "%s"', container_name, directory)
         try:
             result_file = manager.return_backup(files, name, directory, cformat)
         except IOError as error:
             res.update({'error': 'Failed to generate backup: {err}'.format(err=str(error))})
             return res
-        self.sender.send_message('Backup generated in "{}"'.format(result_file))
+        self.sender.send_message('Backup production generated')
         logger.info('Backup generated in "%s"', result_file)
         utils.clean_files(tmp)
         res.update({'result': result_file})
         return res
 
-    def _parse_update_production_log(self, log_path, module):
-        res = {}
-        try:
-            with open(log_path) as log_file:
-                summary = utils.resume_log(log_file)
-        except IOError as error:
-            res.update({'error': 'Could not open log "{log}": {err}'.format(
-                log=log_path, err=utils.get_error_message(error))
-            })
-            return res
-        for log_type, log_num in sorted(summary.items(), key=lambda log: log[0]):
-            logger.info('+-- %s %s', log_type.replace('_', ' ').title(), len(log_num))
-        summary.update({'log_file': log_path})
-        if summary.get('errors') or summary.get('critical'):
-            update_error = 'An error ocurred while updating "{mod}"'.format(mod=module)
-            res.update({'error': update_error})
-            logger.error(update_error)
-        res.update({
-            'summary': summary,
-            'attachment': {
-                'file_name': os.path.basename(log_path),
-                'file': utils.generate_attachment(log_path),
-                'type': 'text/plain'
-            }
-        })
-        return res
-
     def _switch_production_container(self, container_name):
         res = {'result': False}
         manager = self.instance_manager
@@ -1443,7 +1325,7 @@ class CommandV(Extensible):
         logger.debug('Branchesv result:\n%s', cmd_output)
         # Stop instance
         logger.info('Stopping instance in "%s"', container_name)
-        self.sender.send_message('Stopping instance in "{}"'.format(container_name))
+        self.sender.send_message('Stopping instance')
         stopped = manager.ensure_instance_status(
             ensure_running=False, max_tries=self.__config.status_checks
         )
