@@ -2,9 +2,10 @@ import os
 import errno
 import logging
 import simplejson as json
+import deployv_static
 from jinja2 import Template
 from deployv.helpers import json_helper, utils
-from deployv.base.errors import ParameterError
+from deployv.base.errors import ParameterError, NoItemFound
 try:
     from configparser import ConfigParser
 except ImportError:
@@ -13,7 +14,7 @@ except ImportError:
 _logger = logging.getLogger(__name__)
 
 
-class DeployvConfig(object):
+class DeployvConfig:
 
     def __init__(self, worker_config=None, deploy_config=None, load_defaults=True):
         self._extras = {}
@@ -23,8 +24,10 @@ class DeployvConfig(object):
         self.rmq = {}
         self.instance_config = {}
         self.container_config = {}
+        self.maintenance_page = {}
         self.worker_config = self.parse_config(worker_config)
         self._deploy_config = deploy_config or {}
+        self.variables = {}
         self.generate_configuration(self._deploy_config, load_defaults=load_defaults)
 
     def __getattr__(self, name):
@@ -125,8 +128,7 @@ class DeployvConfig(object):
         :rtype: ConfigParser
         """
         config = self.worker_config or ConfigParser()
-        default_config_file = os.path.join(os.path.dirname(__file__),
-                                           '../config/deployv.conf')
+        default_config_file = deployv_static.get_template_path('deployv.conf')
         config_files = [default_config_file]
         main_dirs = ['/etc/deployv', os.path.expanduser('~/.config/deployv')]
         for main_dir in main_dirs:
@@ -140,19 +142,20 @@ class DeployvConfig(object):
                     if os.path.isfile(filepath) and filename.endswith('.conf'):
                         config_files.append(filepath)
         if len(config_files) == 1:
-            _logger.warn("Warning: no config file detected in the config "
-                         "paths. You can create a `deployv.conf` file in "
-                         "`/etc/deployv/` or `~/.config/deployv/` paths. "
-                         "You can also run `deployvcmd setup_config` to "
-                         "automatically create the config files for you.")
+            _logger.warning(
+                "Warning: no config file detected in the config "
+                "paths. You can create a `deployv.conf` file in "
+                "`/etc/deployv/` or `~/.config/deployv/` paths. "
+                "You can also run `deployvcmd setup_config` to "
+                "automatically create the config files for you.")
         if additional_config_file:
             config_files.append(additional_config_file)
         if config_files:
             _logger.info("Loading config files: %s", ", ".join(config_files))
             loaded_files = config.read(config_files)
             if len(config_files) != len(loaded_files):
-                _logger.warn("Error loading config files: %s",
-                             ", ".join(set(config_files) - set(loaded_files)))
+                _logger.warning("Error loading config files: %s",
+                                ", ".join(set(config_files) - set(loaded_files)))
         self.set_worker_config_sections(config)
         return config
 
@@ -164,7 +167,7 @@ class DeployvConfig(object):
         :param config: Worker configuration
         :type config: ConfigParser object
         """
-        booleans = ['use_template', 'use_nginx', 'rmq_ssl']
+        booleans = ['use_template', 'use_nginx', 'rmq_ssl', 'pika_logs']
         integers = ['db_port', 'workers', 'max_instances', 'jobs', 'rmq_port', 'rmq_timeout']
         if not isinstance(config, ConfigParser):
             raise ParameterError('worker_config: Invalid type {}'.format(type(config)))
@@ -210,16 +213,17 @@ class DeployvConfig(object):
         self._deploy_config = res
         self.instance_config = res.get('instance') or {}
         self.container_config = res.get('container_config') or {}
+        self.maintenance_page = res.get('maintenance_page') or {}
         # Use a copy instead of the original dictionary so we don't alter the original
         # config, this will allow us to update the properties without the params that
         # were previously set
         extras = res.get('group_config', {}).copy()
         for param in special_params:
             get_method = '_get_' + param
-            func = getattr(self, get_method)
-            if func:
+            try:
+                func = getattr(self, get_method)
                 new_val = func(res)
-            else:
+            except AttributeError:
                 raise ParameterError('The attribute {param} was defined as a special'
                                      ' attribute but the expected `_get_{param}()` method was'
                                      ' not found'.format(param=param))
@@ -250,8 +254,7 @@ class DeployvConfig(object):
 
     def setup_config_dirs(self):
         _logger.info("Writing config files")
-        default_config_file = os.path.join(os.path.dirname(__file__),
-                                           '../config/deployv.conf')
+        default_config_file = deployv_static.get_template_path('deployv.conf')
         for main_dir in ['/etc/deployv', os.path.expanduser('~/.config/deployv')]:
             config_file_name = os.path.join(main_dir, 'deployv.conf')
             error_msg = "Couldn't write config file: '%s' (skipped)"
@@ -261,7 +264,7 @@ class DeployvConfig(object):
                 os.makedirs(addon_dir)
             except OSError as error:
                 if error.errno != errno.EEXIST:
-                    _logger.warn(error_msg, config_file_name)
+                    _logger.warning(error_msg, config_file_name)
                     continue
             else:
                 if is_user_dir:
@@ -273,7 +276,7 @@ class DeployvConfig(object):
                 with open(config_file_name, 'w+') as config_file:
                     config.write(config_file)
             except (OSError, IOError):
-                _logger.warn(error_msg, config_file_name)
+                _logger.warning(error_msg, config_file_name)
             else:
                 _logger.info("Created config file: '%s'", config_file_name)
                 if is_user_dir:
@@ -286,6 +289,13 @@ class DeployvConfig(object):
         if uid is not None:
             os.chown(path, int(uid), int(gid))
 
+    def get_mainrepo_path(self):
+        repos = self.instance_config.get('repositories', [])
+        main_repo = [repo for repo in repos if repo.get('main_repo')]
+        if main_repo:
+            return main_repo[0].get('path')
+        raise NoItemFound('No main repo found in the config')
+
 
 def get_config(config_file=None):
     config = DeployvConfig(worker_config=config_file)
