@@ -14,28 +14,36 @@ import base64
 import tarfile
 import paramiko
 import csv
-import errno
 from six import string_types, binary_type
 import binascii
 from string import digits, ascii_letters  # pylint: disable=deprecated-module
 from deployv.helpers import container, json_helper
 from deployv.helpers.decompress_helper import DecompressHelper
 from deployv import base
+import deployv_static
 from docker.errors import APIError
-from jinja2 import Template
+from jinja2 import Environment, FileSystemLoader
+from errno import EEXIST
+from deepdiff import DeepDiff
+from stat import S_IREAD, S_IWRITE
 import socket
 import fcntl
 import struct
 import ipaddress
 
 _AVAILABLES = {
-    "production": ['update_production', 'post_process', 'restart_instance', 'restart_container'],
+    "production": ['update_production', 'post_process', 'restart_instance',
+                   'restart_container', 'reload_config'],
     "test": [],
     "updates": [],
     "develop": [],
 }
 
-logger = logging.getLogger(__name__)  # pylint: disable=C0103
+IMPLEMENTED_MSG_ERROR = ('The class you are instantiating does '
+                         'not have this method implemented')
+
+# pylint: disable=C0103
+logger = logging.getLogger(__name__)
 
 
 def copy_list_dicts(lines):
@@ -104,9 +112,8 @@ def load_default_config():
 
     :return: Dictionary with the default configuration
     """
-    default_config_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', 'config')
-    default_config_file = os.path.join(default_config_path, 'default_config.json')
-    default_config = json_helper.load_json(default_config_file)
+    default_config_path = deployv_static.get_template_path('default_config.json')
+    default_config = json_helper.load_json(default_config_path)
     return default_config
 
 
@@ -159,6 +166,8 @@ def clean_files(files):
 
     :param files: A list or a str of absolute or relative paths thar will be erased
     """
+    if not files:
+        return
     items = files if is_iterable(files) else [files]
     for item in items:
         fname = item[0] if is_iterable(item) else item
@@ -239,7 +248,7 @@ def get_decompress_object(file_name):
 
         >>> from deployv.helpers import utils
         >>> utils.get_decompress_object('filename.tar.bz2')
-        (<bound method type.open of <class 'tarfile.TarFile'>>, 'r:bz2')
+        (<bound method TarFile.open of <class 'tarfile.TarFile'>>, 'r:bz2')
 
     Or if you pass a tar.gz file:
 
@@ -247,7 +256,7 @@ def get_decompress_object(file_name):
 
         >>> from deployv.helpers import utils
         >>> utils.get_decompress_object('filename.tar.gz')
-        (<bound method type.open of <class 'tarfile.TarFile'>>, 'r:gz')
+        (<bound method TarFile.open of <class 'tarfile.TarFile'>>, 'r:gz')
 
     :param file_name: File name to be extracted
     :return: Tuple with the object and mode
@@ -435,23 +444,32 @@ def generate_attachment(file_name):
 
 
 def get_error_message(exception_obj):
-    """ Some exceptions uses message and others explanation, so with this little helper we avoid
-        to rewrite the code every single time.
+    """Get the message error from exception object or dict.
 
-    :param exception_obj: The raised exception Object
-    :return: A string containing the exception message
+    :param exception_obj: the exception object or dict exception where get the message.
+    :type: object, dict
+    :return: A string containing the exception message.
+    :rtype: str
     """
     error_attrs = ['stderr_output', 'explanation', 'msg', 'strerror',
-                   'message', 'error_message', 'data']
+                   'message', 'error_message', 'data', 'name']
+
+    if isinstance(exception_obj, str):
+        return str(exception_obj)
     for attr in error_attrs:
-        if hasattr(exception_obj, attr):
-            error_msg = getattr(exception_obj, attr)
-            if not error_msg:
-                continue
-            error_msg = json_helper.load_json(error_msg, False) or error_msg
-            if isinstance(error_msg, dict):
-                error_msg = error_msg.get("error") or error_msg.get("message") or error_msg
-            return error_msg
+        if isinstance(exception_obj, dict) and exception_obj.get(attr):
+            return exception_obj[attr]
+        if not hasattr(exception_obj, attr):
+            continue
+        msg = getattr(exception_obj, attr)
+        if not msg:
+            continue
+        msg = json_helper.load_json(msg, False) or msg
+        if isinstance(msg, dict):
+            msg = msg.get("error") or msg.get("message") or msg
+        if isinstance(msg, bytes):
+            msg = msg.decode()
+        return msg
     return repr(exception_obj)
 
 
@@ -460,7 +478,7 @@ def makedir(path_name):
     try:
         os.makedirs(path_name)
     except OSError as error:
-        if error.errno != errno.EEXIST:
+        if error.errno != EEXIST:
             raise
     return path_name
 
@@ -546,22 +564,6 @@ def encode(string, errors='replace'):
     return string
 
 
-def version_cid(customer_id):
-    """ Extract the version from the customer id
-
-    :param customer_id: A string with the customer id (customer80, customer100)
-    :return: The sversion (8.0, 10.0)
-    """
-    version = None
-    match = re.match("([a-z_]+)([0-9]+|saas-[0-9]+)", customer_id, re.I)
-    if match:
-        version = match.groups()[-1]
-        if version.isdigit():
-            res = float(version) / 10.0
-            version = str(res)
-    return version
-
-
 def parse_url(url):
     """ Parses an url and returns the parts that we are interested in:
         port, domain, user, destination path
@@ -583,17 +585,31 @@ def parse_url(url):
     return res
 
 
-def upload_scp(filename, credentials):
+def upload_scp(filename, credentials, retry=3):
     logger.info('Uploading %s using SFTP', filename)
     port = credentials.get('port') if credentials.get('port') else 22
-    transport = paramiko.Transport((credentials.get('domain'), int(port)))
     private_key = paramiko.RSAKey.from_private_key_file(
         os.path.expanduser(os.path.join('~', '.ssh', 'id_rsa')))
-    transport.connect(username=credentials.get('user'), pkey=private_key)
-    sftp = paramiko.SFTPClient.from_transport(transport)
-    sftp.chdir(credentials.get('folder'))
-    sftp.put(filename, os.path.basename(filename))
-    sftp.close()
+    transport = paramiko.Transport((credentials.get('domain'), int(port)))
+    sftp = None
+    c = 0
+    while c <= retry:
+        try:
+            transport.connect(username=credentials.get('user'), pkey=private_key)
+            sftp = paramiko.SFTPClient.from_transport(transport)
+            sftp.chdir(credentials.get('folder'))
+            sftp.put(filename, os.path.basename(filename))
+            c = retry + 1
+        except paramiko.ssh_exception.SSHException:
+            c += 1
+            if c >= retry:
+                raise
+            logger.warning('Error while uploading the file, retry %s/%s', c, retry)
+        finally:
+            if sftp:
+                sftp.close()
+            if transport:
+                transport.close()
 
 
 def upload_file(file_name, url):
@@ -712,10 +728,8 @@ def verify_free_disk_space(expected_free_space, directory=False):
 
 
 def get_certificates_pac():
-    templates = os.path.join(
-        os.path.dirname(os.path.abspath(__file__)), '..', 'templates')
-    certificate_path = os.path.join(templates, 'files',
-                                    'l10n_mx_edi.certificate.csv')
+    certificate_path = deployv_static.get_template_path(
+        'l10n_mx_edi.certificate.csv')
     try:
         csvfile = open(certificate_path, 'r')
         certificates = list(csv.DictReader(csvfile))[0]
@@ -729,12 +743,11 @@ def render_template(template, values=False):
     returns the rendered template.
     """
     values = values or {}
-    template_path = os.path.join(os.path.dirname(__file__), '..',
-                                 'templates', template)
-    with open(template_path, 'r') as obj:
-        json_template = obj.read()
-    res = json_helper.load_json(Template(json_template).render(values))
-    return res
+    template_path = deployv_static.get_template_path(template)
+    env = Environment(loader=FileSystemLoader(os.path.dirname(template_path)))
+    template = env.get_template(template)
+    rendered_template = template.render(values)
+    return rendered_template
 
 
 def is_base64_encode(string_encode):
@@ -784,7 +797,7 @@ def get_images_path():
     """
     shell = spur.LocalShell()
     res = shell.run(shlex.split('docker info -f "{{.DockerRootDir}}"')).output
-    return res.strip('\n')
+    return res.strip(b'\n')
 
 
 def get_ip_address(ifname):
@@ -792,11 +805,168 @@ def get_ip_address(ifname):
     return socket.inet_ntoa(fcntl.ioctl(
         tmp_sock.fileno(),
         0x8915,
-        struct.pack('256s', ifname[:15])
+        struct.pack('256s', encode(ifname[:15]))
     )[20:24])
 
 
 def is_docker_ip(ip_addr):
     """Recevives an ip and checks if it is in the docker network range"""
     docker0 = re.sub(r"\.\d{1,3}$", ".0/24", get_ip_address('docker0'))
-    return ipaddress.ip_address(unicode(ip_addr)) in ipaddress.ip_network(unicode(docker0))
+    return ipaddress.ip_address(decode(ip_addr)) in ipaddress.ip_network(decode(docker0))
+
+
+def find_files(file_path, file_name):
+    """Searches for files with the provided name in the provided path and
+    return the list (empty if none is found)
+
+    :param file_path: Path where the search will be done
+    :param file_name: Name of the file to find
+    :return: The contents of the file or False if the file was not found
+    """
+    shell = spur.LocalShell()
+    file_path = shell.run(['find', file_path, '-name', file_name])
+    res = []
+    if file_path:
+        lines = file_path.output.decode('utf8').split('\n')
+        res = [l.strip() for l in lines if l.strip()]
+    return res
+
+
+def compare_repositories(repositories, new_repositories):
+    """Compare the `new_repositories` with the `repositories`, it compare with
+    the `path` where store the repositories, it returns the repositories
+    that have any differentiate or the repository doesn't found in `repositories`.
+
+    :param repositories: The repositories that will compare with `new_repositories`.
+    :type: list
+    :param new_repositories: The repositories that will compare with `repositories`.
+    :type: list
+    :return: The repositories that don't belong in `repositories` or have some diff.
+    :rtype: list
+    """
+    repos_paths = {repo.get('path'): repo for repo in repositories}
+    repos_diff = []
+    exclude_keys = ["root['main_repo']", "root['depth']", "root['is_dirty']"]
+    for new_repo in new_repositories:
+        repo = repos_paths.pop(new_repo.get('path'), {})
+        if not repo:
+            repos_diff.append(new_repo)
+            continue
+        for key, value in new_repo.items():
+            if key == 'repo_url':
+                value.update({'origin': encode(value.get('origin'))})
+                continue
+            if key == 'commit':
+                new_repo.update({key: encode(value[:7])})
+                repo.update({key: repo.get(key)[:7]})
+                continue
+            new_repo.update({key: encode(value)})
+        diff = DeepDiff(new_repo, repo, exclude_paths=exclude_keys)
+        if diff.get('type_changes'):
+            repos_diff.append(new_repo)
+    return repos_diff
+
+
+def parse_variables_to_json(variables):
+    """Convert to dict the env variables used to execute a process in the instance.
+
+    :param variables: the enviroment variables used to build/create/restore
+        and other things that can do with the instance.
+    :type: str
+    :return: a dict with the information getted in `variables`.
+    :rtype: dict
+    """
+    if not variables:
+        return {}
+    if isinstance(variables, list):
+        vs = [v.replace('export', '').strip() for v in variables]
+    else:
+        vs = variables.replace('export', '').strip().split('\n')
+    return container.parse_env_vars(vs)
+
+
+def add_repo(branch, name, repo_path, repo_url, commit=False, config=False,
+             overwrite_repos=False, extra=False):
+    """Creates a dictionary with the info of the specified repository and
+    appends it to the repositories list in the config dictionary or
+    return the dict created.
+
+    :param config: Config dict where the new repo will be added
+    :param name: Name of the repository
+    :param path: Path inside ~/instance where the repository will be cloned
+    :param repo_url: ssh or https URL used to clone the repo
+    :param branch: branch of the repo we want to clone
+    :param commit: the commit of repository
+    :param overwrite_repos: Identificator to overwrite the repositories.
+    :type overwrite_repos: bool
+    :return: Config dict with the new repo
+    """
+    repo = {
+        'branch': branch, 'name': name, 'path': repo_path,
+        'commit': commit or '', 'depth': 1,
+        'repo_url': {'origin': repo_url}
+    }
+    repo.update(extra or {})
+    if not config:
+        return repo
+    if overwrite_repos:
+        config.get('instance', {}).update({'repositories': [repo]})
+    else:
+        config.get('instance', {}).get('repositories', []).append(repo)
+    return config
+
+
+def parse_repo_url(repo_url):
+    """Parses the provided url to get its parts such as the protocol,
+    the org, the repo name, and the domain. The url can be in either format:
+
+    https://github.com/organization/repo
+    git@github.com:organization/repo.git
+
+    :param repo_url: URL to be parsed
+    :type: string
+    :return: dictionary with the parts of the url, if the url contains
+        git@ instead of a proper protocol, https:// will be returned
+        as the protocol
+    """
+    if not repo_url:
+        return {}
+    regex = (
+        r'(?P<protocol>git@|http[s]?:\/\/)(?P<domain>[\.\w\d-]*)[:\/]{1}'
+        r'(?P<repo>(?P<namespace>[\w\d-]*)/(?P<repo_name>[\w\d-]*))(\.git)?'
+    )
+    res = re.match(regex, repo_url)
+    if not res:
+        return {}
+    values = res.groupdict()
+    if values['protocol'] == 'git@':
+        values.update({'protocol': 'https://'})
+    return values
+
+
+def read_lines(filename):
+    """Returns a file content as a list, no blank lines and stipped
+
+    :param filename: Fill path to the file
+    :return: A list with the file content, empty if no lines
+    """
+    with open(filename, 'r') as obj:
+        file_contents = obj.read()
+    lines = file_contents.split('\n')
+    res = [l.strip() for l in lines if l.strip()]
+    return res
+
+
+def deploy_key(ssh_key, folder_path):
+    """Deploy a SSH private key into the temporal working folder.
+    """
+    if not is_base64_encode(ssh_key):
+        logger.warning("Can not use the specified ssh key. It is not encoded in base64")
+        return False
+    logger.info('Deploying keys')
+    ssh_key = decode(base64.b64decode(ssh_key))
+    filename = os.path.join(folder_path, 'id_rsa')
+    with open(filename, 'w') as key_file:
+        key_file.write(ssh_key)
+    os.chmod(filename, S_IREAD + S_IWRITE)
+    return filename
