@@ -84,7 +84,7 @@ import shutil
 import time
 import psycopg2
 from psycopg2 import sql as psycopg2_sql
-from docker.errors import APIError
+from docker.errors import APIError, ContainerError
 import simplejson as json
 import deployv_static
 from passlib.context import CryptContext
@@ -416,37 +416,170 @@ class InstanceV(DockerV):
         logger.info('Logger was saved to: "%s"', working_path)
         return resume
 
-    def update_db(self, modules, db_name):
-        """ Executes "-u module" or "-u module1,module2" on the selected database and returns
-            resumed log and full path to the generated log. See
-            :func:`~dockerv.helpers.utils.resume_log` for detailed information about
-            the return format
+    def update_db(self, db_name, modules=False):
+        """ Updates the database using one of the possible methods of updating.
+
+        Possible methods:
+            - cou: uses click-odoo-update to update the modules that changed in the db.
+            - normal: updates the provided modules or the ones specified in the json.
 
-        :param modules: Module (str) o modules (list) to be updated
         :param db_name: Database name to be updated
+        :param modules: List of modules to update, only used by the normal method.
         :return: Resumed log and full path to the generated file
         """
-        if utils.is_iterable(modules):
-            modules = modules[0]
-        logger.info('Updating database %s, modules %s', db_name, modules)
-        command_odoo = self.odoo_binary
-        if self.config.get('env_vars'):
-            env_vars = self.config.get('env_vars')
-            if env_vars.get('odoo_home') and env_vars.get('odoo_user'):
-                odoo_home = env_vars.get('odoo_home')
-                odoo_user = env_vars.get('odoo_user')
-        else:
-            odoo_home = self.docker_env.get('odoo_home')
-            odoo_user = self.docker_env.get('odoo_user')
-        update_command = ('su {user} -c "{home}/instance/odoo/{odoo}'
-                          ' -d {database} -u {modules} --stop-after-init"') \
-            .format(home=odoo_home, user=odoo_user, database=db_name, modules=modules,
-                    odoo=command_odoo)
-        skip_modules = ['web_environment_ribbon_isolated']
-        skip_summary = all(module in skip_modules for module in modules.split(','))
-        attachment_name = 'update_{mod}_{timestamp}.log'.format(
-            timestamp=utils.get_strtime(), mod=modules)
-        return self.run_and_log(update_command, attachment_name, skip_summary)
+        if not self.__full_config.update_method:
+            self.__full_config.update_method = 'normal'
+        update_methods = {
+            'cou': self._cou_update,
+            'normal': self._module_update
+        }
+        if self.__full_config.update_method not in update_methods:
+            return {'error': 'Unknown update method: %s' % self.__full_config.update_method}
+        update_method = update_methods[self.__full_config.update_method]
+        res = update_method(db_name)
+        return res
+
+    def _module_update(self, db_name):
+        """ Updates the specified database by executing -u module for each module in
+        the json.
+
+        :param db_name: Name of the database to update.
+        :return: dict with the logs of the update and the summary of the results.
+        """
+        res = {'result': {'update_logs': []}, 'attachments': []}
+        image_name = self.config.get('image_name') or self.basic_info.get('image_name')
+        modules = self.__full_config.instance_config.get('update_module') or []
+        logger.info('Updating database %s with the image %s, modules %s',
+                    db_name, image_name, modules)
+        working_folder = path.join(self.config.get("working_folder"),
+                                   self.config.get('container_name'))
+        db_config = self.db_config
+        update_command = (
+            "/entry_point.py run --user odoo '/home/odoo/instance/odoo/{odoo}"
+            " -c /home/odoo/.openerp_serverrc"
+            " -d {database} -u {{module}} --logfile {{log}} --stop-after-init'"
+            .format(odoo=self.odoo_binary, database=db_name))
+        pg_shell = PostgresShell(utils.odoo2postgres(db_config))
+        for module in modules:
+            for attempt in range(2):
+                log_file = 'update_{mod}_{date}.log'.format(mod=module, date=utils.get_strtime())
+                log_path = path.join('/tmp/deployvlogs/', log_file)
+                local_path = path.expanduser(path.join(working_folder, log_path.strip('/')))
+                command = update_command.format(module=module, log=log_path)
+                logger.info('Updating "%s"', module)
+                # Terminate connections before updating
+                pg_shell.terminate_sessions(db_name, db_config.get('db_owner'),
+                                            db_config.get('db_owner_passwd'))
+                try:
+                    self.exec_cmd_image(command, self.config.get('volumes'),
+                                        working_folder, image_name)
+                except (ContainerError, APIError) as error:
+                    res.update({'error': 'Error updating "{mod}": {err}'.format(
+                        mod=module, err=utils.get_error_message(error))})
+                    parsed_log = self._parse_update_log(local_path, module)
+                    summary = parsed_log.get('summary')
+                    if summary:
+                        res['result']['update_logs'].append(summary)
+                        res['attachments'].append(parsed_log.get('attachment'))
+                    return res
+                logger.info('Log was saved to "%s"', local_path)
+                parsed_log = self._parse_update_log(local_path, module)
+                if parsed_log.get('error') and not parsed_log.get('summary'):
+                    # Return the error message if the log file couldn't be generated
+                    res.update(parsed_log)
+                    return res
+                summary = parsed_log.get('summary')
+                if parsed_log.get('error'):
+                    if attempt:
+                        res.update({'error': parsed_log.get('error')})
+                        res['result']['update_logs'].append(summary)
+                        res['attachments'].append(parsed_log.get('attachment'))
+                        return res
+                    logger.info('An error occured, trying again')
+                    continue
+                res['result']['update_logs'].append(summary)
+                res['attachments'].append(parsed_log.get('attachment'))
+                break
+        return res
+
+    def _cou_update(self, db_name):
+        """ Updates the specified database by executing the click-odoo-update script
+        from the OCA, which automatically detects the modules that changed and updates them.
+
+        :param db_name: Name of the database to update.
+        :return: dict with the logs of the update and the summary of the results.
+        """
+        res = {'result': {'update_logs': []}, 'attachments': []}
+        image_name = self.config.get('image_name') or self.basic_info.get('image_name')
+        working_folder = path.join(self.config.get("working_folder"),
+                                   self.config.get('container_name'))
+        logger.info('Updating database %s using COU', db_name)
+        update_command = "/entry_point.py cou -d {database}".format(database=db_name)
+        local_log_path = path.expanduser(
+            path.join(working_folder, 'tmp/deployvlogs/cou_update.log'))
+        pg_shell = PostgresShell(utils.odoo2postgres(self.db_config))
+        for attempt in range(2):
+            # Terminate connections before updating
+            pg_shell.terminate_sessions(db_name, self.db_config.get('db_owner'),
+                                        self.db_config.get('db_owner_passwd'))
+            try:
+                self.exec_cmd_image(update_command, self.config.get('volumes'),
+                                    working_folder, image_name)
+            except (ContainerError, APIError) as error:
+                res.update({'error': 'Error updating using COU: {err}'.format(
+                    err=utils.get_error_message(error))})
+                parsed_log = self._parse_update_log(local_log_path, 'COU')
+                summary = parsed_log.get('summary')
+                if summary:
+                    res['result']['update_logs'].append(summary)
+                    res['attachments'].append(parsed_log.get('attachment'))
+                return res
+            logger.info('Log was saved to "%s"', local_log_path)
+            parsed_log = self._parse_update_log(local_log_path, 'COU')
+            if parsed_log.get('error') and not parsed_log.get('summary'):
+                # Return the error message if the log file couldn't be generated
+                res.update(parsed_log)
+                return res
+            summary = parsed_log.get('summary')
+            if parsed_log.get('error'):
+                if attempt:
+                    res.update({'error': parsed_log.get('error')})
+                    res['result']['update_logs'].append(summary)
+                    res['attachments'].append(parsed_log.get('attachment'))
+                    return res
+                logger.info('An error occured, trying again')
+                continue
+            res['result']['update_logs'].append(summary)
+            res['attachments'].append(parsed_log.get('attachment'))
+            break
+        return res
+
+    def _parse_update_log(self, log_path, module):
+        res = {}
+        try:
+            with open(log_path) as log_file:
+                summary = utils.resume_log(log_file)
+        except IOError as error:
+            res.update({'error': 'Could not open log "{log}": {err}'.format(
+                log=log_path, err=utils.get_error_message(error))
+            })
+            return res
+        for log_type, log_num in sorted(summary.items(), key=lambda log: log[0]):
+            logger.info('+-- %s %s', log_type.replace('_', ' ').title(), len(log_num))
+        summary.update({'log_file': log_path})
+        if summary.get('errors') or summary.get('critical'):
+            update_error = 'An error ocurred while updating "{mod}"'.format(mod=module)
+            res.update({'error': update_error})
+            logger.error(update_error)
+        res.update({
+            'summary': summary,
+            'attachment': {
+                'file_name': path.basename(log_path),
+                'file': utils.generate_attachment(log_path),
+                'type': 'text/plain'
+            }
+        })
+        return res
 
     def install_module(self, modules, db_name, without_demo=True):
         """ Executes "-i module" or "-i module1,module2" on the selected database and returns
@@ -658,7 +791,9 @@ class InstanceV(DockerV):
         if not dump_name:
             logger.error('Database could not be dumped')
             return files
-        files.append(dump_name)
+        files.append(
+            dump_name if not path.isdir(dump_name) else (dump_name, path.basename(dump_name))
+        )
         try:
             manifest_path = self._get_db_manifest(tmp_dir)
             files.append(manifest_path)
